#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
const xpath_1 = __importDefault(require("xpath"));
const xmldom_1 = require("xmldom");
const operator_1 = require("./operator");
const loader_1 = __importDefault(require("./dom/loader"));
const store_1 = __importDefault(require("./dom/store"));
const debug = util_1.default.debuglog('junit-reports-combiner');
const argv = process.argv.slice(2)
    .map(arg => {
    const matches = arg.match(/^--(\w[\w-]*)(?:|\s*=\s*(.*)\s*)$/);
    return matches === null ? ['--', arg] : [matches[1], matches[2]];
})
    .reduce((map, [key, value]) => map.set(key, (map.get(key) || []).concat(value)), new Map());
const namespaces = (argv.get('namespace') || [])
    .map(namespace => namespace.split(/\s*=\s*/))
    .reduce((namespaces, [key, value]) => {
    namespaces[key] = value;
    return namespaces;
}, {});
const treeStructure = [
    ['.//testsuite', (argv.get('test-suite-key') || [])[0] || 'string(@name)'],
    ['.//testcase', (argv.get('test-case-key') || [])[0] || 'concat(@classname,"#",@name)'],
    ['.//failure|.//error', (argv.get('test-result-key') || [])[0] || 'string(.)']
];
const Operator = (argv.get('operator') || [])
    .map((operator) => {
    switch (operator) {
        case 'union': return operator_1.Union;
        case 'intersect': return operator_1.Intersect;
        case 'except': return operator_1.Except;
        default: throw new TypeError();
    }
})
    .reduce((_, operator) => operator, operator_1.Union);
const distinct = !(((values) => {
    if (values === undefined)
        return undefined;
    if (values[0] === undefined)
        return true;
    return Boolean(values[0]);
})(argv.get('all')) || false);
debug(util_1.default.inspect({ Operator, distinct, namespaces, treeStructure }));
const operator = new Operator(true, { namespaces, treeStructure });
const files = ((files) => files.length > 1 ? files : files.concat(0))(argv.get('--') || []);
const first = files[0];
const others = files.slice(1);
debug(util_1.default.inspect({ first, others }));
others
    .reduce(async (previous, file) => {
    debug(file.toString());
    const document = ((rootNodeName, namespaces) => {
        const document = new xmldom_1.DOMImplementation().createDocument(null, null, null);
        document.appendChild(document.createProcessingInstruction('xml', 'version="1.0" encoding="UTF-8"'));
        const root = document.createElement(rootNodeName);
        for (const [key, value] of Object.entries(namespaces))
            root.setAttribute(`xmlns:${key}`, value);
        document.appendChild(root);
        return document;
    })('testsuites', namespaces);
    const curent = await new loader_1.default().load(file);
    operator.operation(await previous, curent, document.documentElement);
    return document;
}, new loader_1.default().load(first))
    .then(document => {
    const select = xpath_1.default.useNamespaces(namespaces);
    select('.//testsuite', document)
        .filter((node) => node.nodeType === 1 /* Node.ELEMENT_NODE */)
        .forEach(testSuite => {
        const failures = select('count(.//failure)', testSuite, true);
        const errors = select('count(.//error)', testSuite, true);
        testSuite.setAttribute('failures', failures.toString());
        testSuite.setAttribute('errors', errors.toString());
    });
    return document;
})
    .then(document => new store_1.default(process.stdout).store(document))
    .catch(error => {
    console.error(error);
    process.exit(1);
});
//# sourceMappingURL=cli.js.map