"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const util_1 = __importDefault(require("util"));
const xmldom_1 = require("xmldom");
class DOMLoader {
    constructor(parser = new xmldom_1.DOMParser()) {
        this.parser = parser;
    }
    static readEncodingName(buffer) {
        const declarationStartBytes = Buffer.from('<?xml');
        const declarationEndBytes = Buffer.from('?>');
        if (declarationStartBytes.compare(buffer, 0, declarationStartBytes.length) !== 0)
            return undefined;
        const index = buffer.indexOf(declarationEndBytes);
        if (index === -1)
            return undefined;
        const declaration = buffer.toString('utf8', 0, index + declarationEndBytes.length);
        const matches = declaration.match(/\bencoding\s*=\s*(["'])([A-Za-z][A-Za-z0-9._-]*)\1/);
        return matches ? matches[2] : undefined;
    }
    static parseFromBuffer(parser, decoder, buffer, mimeType) {
        const xml = decoder.decode(buffer);
        return parser.parseFromString(xml, mimeType);
    }
    async load(path) {
        const buffer = await util_1.default.promisify(fs_1.default.readFile)(path);
        const encoding = DOMLoader.readEncodingName(buffer);
        return DOMLoader.parseFromBuffer(this.parser, new util_1.default.TextDecoder(encoding), buffer, 'application/xml');
    }
}
exports.default = DOMLoader;
//# sourceMappingURL=loader.js.map