"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = __importDefault(require("util"));
const xpath_1 = __importDefault(require("xpath"));
const debug = util_1.default.debuglog('junit-reports-combiner');
class Operator {
    constructor(distinct = true, options = {}) {
        this.distinct = distinct;
        this.namespaces = options.namespaces || {};
        this.treeStructure = options.treeStructure || [];
        this.select = xpath_1.default.useNamespaces(this.namespaces);
    }
    operation(node1, node2, sink) {
        const structureHashTree = (node, treeStructure) => {
            if (treeStructure.length === 0)
                return new Map();
            const [elementSelector, keySelector] = treeStructure[0];
            const entries = this.select(elementSelector, node)
                .map(element => [
                this.select(keySelector, element, true),
                [element, structureHashTree(element, treeStructure.slice(1))]
            ]);
            return entries.reduce((map, [key, value]) => map.set(key, (map.get(key) || []).concat([value])), new Map());
        };
        const confront = (source, tree, sink, treeStructure) => {
            var _a;
            if (treeStructure.length === 0) {
                if (this.needToRetain(true, true)) {
                    Array.from(source.childNodes).forEach(node => sink.appendChild(node.cloneNode(true)));
                }
                else {
                    (_a = sink.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(sink);
                }
                return;
            }
            const [elementSelector, keySelector] = treeStructure[0];
            const elements = this.select(elementSelector, source);
            const keys = elements.map(element => {
                const key = this.select(keySelector, element, true);
                const entrires = tree.get(key) || [];
                const matches = entrires.length > 0;
                if (matches) {
                    const clone = element.cloneNode(false);
                    sink.appendChild(clone);
                    entrires.forEach(([, map]) => confront(element, map, clone, treeStructure.slice(1)));
                }
                else if (this.needToRetain(true, false)) {
                    const clone = element.cloneNode(true);
                    sink.appendChild(clone);
                }
                return matches ? key : null;
            }).filter((key) => key !== null);
            if (this.needToMergeOther())
                for (const [key, values] of tree) {
                    if (keys.includes(key))
                        continue;
                    for (const [node] of values)
                        sink.appendChild(node.cloneNode(true));
                }
        };
        confront(node2, structureHashTree(node1, this.treeStructure), sink, this.treeStructure);
        return sink;
    }
}
class Union extends Operator {
    needToRetain(oneContains, otherContains) {
        return oneContains || otherContains;
    }
    needToMergeOther() {
        return true;
    }
}
exports.Union = Union;
class Intersect extends Operator {
    needToRetain(oneContains, otherContains) {
        return oneContains && otherContains;
    }
    needToMergeOther() {
        return false;
    }
}
exports.Intersect = Intersect;
class Except extends Operator {
    needToRetain(oneContains, otherContains) {
        return oneContains && !otherContains;
    }
    needToMergeOther() {
        return false;
    }
}
exports.Except = Except;
//# sourceMappingURL=operator.js.map